@@ -2,7 +2,9 @@
 
 import os
 import json
+import random
 import logging
+from copy import deepcopy
 
 from tornado import gen
 
@@ -26,6 +28,7 @@ class F(object):
     source_path = "s"
     target_path = "t"
     id = "id"
+    replica = "r"
 
 
 class C(object):
@@ -35,6 +38,8 @@ class C(object):
     rename = "r"
     move = "m"
     copy = "cp"
+    update_replica = "ur"
+    update_file_info = "ufi"
 
 
 class InvalidValueError(Exception):
@@ -82,15 +87,25 @@ class FileSystemTree(object):
             cls._instance.tree = {F.children: {}, F.type: "root"}
             cls._instance.files = {}
             cls._instance.editlog = None
-            cls._instance.load_fsimage()
-            cls._instance.load_editlog()
-            cls._instance.dump_fsimage()
-            cls._instance.editlog = AppendLogJson(os.path.join(CONFIG["data_path"], "editlog"))
+            cls._instance.status = "booting"
         return cls._instance
 
     @classmethod
     def instance(cls):
-        return cls._instance
+        if cls._instance and cls._instance.status == "ready":
+            return cls._instance
+        else:
+            return None
+
+    @gen.coroutine
+    def recover(self):
+        self.status = "recovering"
+        yield self.load_fsimage()
+        yield self.load_editlog()
+        yield self.dump_fsimage()
+        self.editlog = AppendLogJson(os.path.join(CONFIG["data_path"], "editlog"))
+        # TODO: synchronize between name node and data nodes
+        self.status = "ready"
 
     def create(self, file_path, file_info):
         result = False
@@ -119,10 +134,7 @@ class FileSystemTree(object):
                 del self.files[file_id]
                 task = {"command": "delete", "name": file_id}
                 for i in Connection.id_decompress:
-                    if i not in Connection.tasks:
-                        Connection.tasks[i] = [task]
-                    else:
-                        Connection.tasks[i].append(task)
+                    Connection.push_task(i, task)
             elif file[F.type] == F.dir:
                 self.delete_files(file)
             if self.editlog:
@@ -141,10 +153,7 @@ class FileSystemTree(object):
                 del self.files[file_id]
                 task = {"command": "delete", "name": file_id}
                 for i in Connection.id_decompress:
-                    if i not in Connection.tasks:
-                        Connection.tasks[i] = [task]
-                    else:
-                        Connection.tasks[i].append(task)
+                    Connection.push_task(i, task)
                 LOG.debug("delete file: %s", file)
                 yield gen.moment
 
@@ -175,6 +184,78 @@ class FileSystemTree(object):
             raise InvalidValueError("invailed file name: %s" % new_name)
         return result
 
+    def update_file_info(self, file_path, file_info):
+        result = False
+        exists, file_type, file, parent = self.get_info(file_path)
+        if exists:
+            if file_type == F.file:
+                file_id = file[F.id]
+                self.files[file_id] = file_info
+                result = True
+            else:
+                raise InvalidValueError("must by file not directory: %s" % file_path)
+        else:
+            raise FileNotExistsError("file not exists: %s" % file_path)
+        return result
+
+    @gen.coroutine
+    def update_replica(self, file_path, replica):
+        result = False
+        exists, file_type, file, parent = self.get_info(file_path)
+        if exists:
+            if file_type == F.file:
+                file_id = file[F.id]
+                file_info = self.files[file_id]
+                file_info["replica"] = replica
+                data_nodes = Connection.get_node_infos()
+                data_node_ids = list(data_nodes.keys())
+                if (
+                        (file_info["replica"] > file_info["current_replica"] and len(data_nodes) >= file_info["current_replica"]) or
+                        (file_info["replica"] < file_info["current_replica"] and len(data_nodes) >= file_info["replica"])
+                    ):
+                    for block in file_info["blocks"]:
+                        old_node_ids = []
+                        new_node_ids = []
+                        for node_id in data_node_ids:
+                            if node_id not in block[2]:
+                                new_node_ids.append(node_id)
+                            else:
+                                old_node_ids.append(node_id)
+                        if old_node_ids:
+                            delta = file_info["replica"] - len(old_node_ids)
+                            block[2] = []
+                            block[2].extend(old_node_ids)
+                            if delta > 0: # increase block replica
+                                if len(new_node_ids) <= delta:
+                                    block[2].extend(new_node_ids)
+                                else:
+                                    new_node_ids = random.sample(new_node_ids, delta)
+                                    block[2].extend(new_node_ids)
+                                source_node_id = random.choice(old_node_ids)
+                                task = {"command": "replicate", "name": file_id, "block": block[0], "ids": new_node_ids}
+                                Connection.push_task(source_node_id, task)
+                            elif delta < 0: # decrease block replica
+                                delta = -delta
+                                if len(block[2]) > delta:
+                                    delete_node_ids = random.sample(block[2], delta)
+                                    block[2] = [i for i in block[2] if i not in delete_node_ids]
+                                    for delete_node_id in delete_node_ids:
+                                        task = {"command": "delete", "name": file_id, "block": block[0]}
+                                        Connection.push_task(delete_node_id, task)
+                        yield gen.moment
+                    file_info["current_replica"] = replica
+                    for block in file_info["blocks"]:
+                        if len(block[2]) < file_info["current_replica"]:
+                            file_info["current_replica"] = len(block[2])
+                result = True
+                if self.editlog:
+                    self.editlog.writeline({F.cmd: C.update_file_info, F.path: file_path, F.info: file_info})
+            else:
+                raise InvalidValueError("must by file not directory: %s" % file_path)
+        else:
+            raise FileNotExistsError("file not exists: %s" % file_path)
+        return result
+
     def is_valid_name(self, name):
         result = True
         if "/" in name:
@@ -314,29 +395,40 @@ class FileSystemTree(object):
                 result[3] = parent
         return result
 
+    @gen.coroutine
     def load_fsimage(self):
         result = False
         try:
             LOG.info("loading fsimage ...")
             fsimage_path = os.path.join(CONFIG["data_path"], "fsimage")
             fsimage = AppendLogJson(fsimage_path)
+            n = 0
             for line in fsimage.iterlines():
+                if n >= 100:
+                    n = 0
+                    yield gen.moment
                 if line[F.cmd] == C.create:
                     self.create(line[F.path], line[F.info])
                 elif line[F.cmd] == C.makedirs:
                     self.makedirs(line[F.path])
+                n += 1
             result = True
         except Exception as e:
             LOG.exception(e)
         return result
 
+    @gen.coroutine
     def load_editlog(self):
         result = False
         try:
             LOG.info("loading editlog ...")
             editlog_path = os.path.join(CONFIG["data_path"], "editlog")
             editlog = AppendLogJson(editlog_path)
+            n = 0
             for line in editlog.iterlines():
+                if n >= 100:
+                    n = 0
+                    yield gen.moment
                 if line[F.cmd] == C.create:
                     self.create(line[F.path], line[F.info])
                 elif line[F.cmd] == C.makedirs:
@@ -349,11 +441,15 @@ class FileSystemTree(object):
                     self.move(line[F.source_path], line[F.target_path])
                 elif line[F.cmd] == C.copy:
                     self.copy(line[F.source_path], line[F.target_path])
+                elif line[F.cmd] == C.update_file_info:
+                    self.update_file_info(line[F.path], line[F.info])
+                n += 1
             result = True
         except Exception as e:
             LOG.exception(e)
         return result
 
+    @gen.coroutine
     def dump_fsimage(self):
         result = False
         try:
@@ -364,7 +460,7 @@ class FileSystemTree(object):
             editlog_path = os.path.join(CONFIG["data_path"], "editlog")
             LOG.debug("new fsimage: %s", new_fsimage_path)
             self.new_fsimage = AppendLogJson(new_fsimage_path)
-            self.dump_files("/", self.tree)
+            yield self.dump_files("/", self.tree)
             if os.path.exists(old_fsimage_path):
                 os.remove(old_fsimage_path)
             if os.path.exists(fsimage_path):
@@ -378,11 +474,12 @@ class FileSystemTree(object):
             LOG.exception(e)
         return result
 
+    @gen.coroutine
     def dump_files(self, file_path, file):
         if F.children in file and file[F.children]:
             for name in file[F.children]:
                 child = file[F.children][name]
-                self.dump_files(os.path.join(file_path, name), child)
+                yield self.dump_files(os.path.join(file_path, name), child)
         else:
             if file[F.type] == F.file:
                 file_id = file[F.id]
