@@ -43,6 +43,10 @@ parser_file_rename = subparsers_file.add_parser("rename", help = "rename file")
 parser_file_rename.add_argument("-r", "--remote-path", required = True, help = "remote file path", default = "")
 parser_file_rename.add_argument("-n", "--new-name", required = True, help = "new file name", default = "")
 
+parser_file_update = subparsers_file.add_parser("update", help = "update file")
+parser_file_update.add_argument("-r", "--remote-path", required = True, help = "remote file path", default = "")
+parser_file_update.add_argument("-R", "--replica", required = True, help = "replica count", type = int, default = 1)
+
 parser_file_download = subparsers_file.add_parser("download", help = "download file")
 parser_file_download.add_argument("-l", "--local-path", required = True, help = "local file path", default = "")
 parser_file_download.add_argument("-r", "--remote-path", required = True, help = "remote file path", default = "")
@@ -71,6 +75,13 @@ parser_directory_rename.add_argument("-n", "--new-name", required = True, help =
 parser_directory_list = subparsers_directory.add_parser("list", help = "list directory's children")
 parser_directory_list.add_argument("-r", "--remote-path", required = True, help = "remote directory path", default = "")
 
+# operate with cluster
+parser_cluster = subparsers.add_parser("cluster", help = "operate with cluster API")
+subparsers_cluster = parser_cluster.add_subparsers(dest = "operation", help = 'sub-command cluster help')
+
+parser_cluster_info = subparsers_cluster.add_parser("info", help = "cluster's info")
+parser_cluster_info.add_argument("-r", "--raw", help = "display raw json data", action = "store_true")
+
 args = parser.parse_args()
 
 
@@ -212,6 +223,18 @@ def main():
                                 print("rename file[%s] to %s failed: %s" % (args.remote_path, args.new_name, data["result"]))
                         else:
                             print("error:\ncode: %s\ncontent: %s" % (r.status_code, r.content))
+                elif operation == "update":
+                    if args.remote_path and args.replica:
+                        json_data = {"path": args.remote_path, "replica": args.replica}
+                        r = requests.put(url, json = json_data)
+                        if r.status_code == 200:
+                            data = r.json()
+                            if "result" in data and data["result"] == "ok":
+                                print("update file[%s] success" % args.remote_path)
+                            else:
+                                print("update file[%s] failed: %s" % (args.remote_path, data["result"]))
+                        else:
+                            print("error:\ncode: %s\ncontent: %s" % (r.status_code, r.content))
                 elif operation == "download":
                     if not os.path.exists(args.local_path):
                         success = True
@@ -332,7 +355,44 @@ def main():
                                 print("list directory[%s] failed: %s" % (args.remote_path, data["result"]))
                         else:
                             print("error:\ncode: %s\ncontent: %s" % (r.status_code, r.content))
-
+            elif object == "cluster":
+                if operation == "info":
+                    r = requests.get(url)
+                    if r.status_code == 200:
+                        data = r.json()
+                        if args.raw:
+                            print(json.dumps(data, indent = 4, sort_keys = True))
+                        else:
+                            if data["result"] == "ok":
+                                print("online nodes:")
+                                print_table_result(
+                                    data["info"]["online_nodes"],
+                                    [
+                                        "id",
+                                        "node_id",
+                                        "http_host",
+                                        "http_port",
+                                        "data_path",
+                                    ]
+                                )
+                                print("\noffline nodes:")
+                                print_table_result(
+                                    data["info"]["offline_nodes"],
+                                    [
+                                        "id",
+                                        "node_id",
+                                        "http_host",
+                                        "http_port",
+                                        "data_path",
+                                    ]
+                                )
+                            else:
+                                print_table_result(
+                                    [data],
+                                    ["result", "message"]
+                                )
+                    else:
+                        print("error:\ncode: %s\ncontent: %s" % (r.status_code, r.content))
                         
     except Exception as e:
         logging.error(logging.traceback.format_exc())
